import os, sys
import subprocess
import netCDF4
import time
import calendar

REFTIMEUTC  = calendar.timegm((2000,1,1,0,0,0))

# WARNING: This script currently presumes data files are netCDF only.
# Needs modification for any other formats.

# User should specifiy directory with new files here
datadir = "data"

# Modify descriptions of each data type as needed
descriptions = {}
descriptions["LIDARPC_L2"]          = "Lidar point cloud data in LAS format."
descriptions["LIDARWSEM_L2"]        = "Lidar water surface elevation model in raster format."
descriptions["LIDARPROFILE_L2"]     = "Lidar data averaged in time and cross-track but not on a raster grid."
descriptions["LIDARPROFILE250_L2"]  = "Lidar data averaged at 250 m in time and cross-track but not on a raster grid."
descriptions["LIDARPROFILE2000_L2"] = "Lidar data averaged at 2000 m in time and cross-track but not on a raster grid."
descriptions["MASSVIS_L2"]          = "Modular Aerial Sensing System visible imagery in geotiff format."
descriptions["MASSVNIR_L2"]         = "Modular Aerial Sensing System visible+near infrared imagery in BIL format."
descriptions["MASSLWIR_L2"]         = "Modular Aerial Sensing System long-wavelength infrared imagery in geotiff format."
descriptions["WAVESPECTRA_L2"]      = "Modular Aerial Sensing System wave spectra and bulk statistics in netCDF format."
descriptions["RADSST_L2"]           = "Along-track sea surface temperature in netCDF format."

# Get listing of new files
datafiles = os.listdir(datadir)

# Loop through each data file to generate corresponding xml file
for datafile in datafiles:
    if datafile[-3:] != ".nc":
        continue
    infile = os.path.join(datadir,datafile)
    print(infile)

    fileidentifier    = datafile[:-3]
    datadate          = "TBD"

    # Compute checksum for data file
    md5sumCmd = ['md5sum', infile]
    proc = subprocess.run(md5sumCmd, stdout=subprocess.PIPE)
    md5CheckSum = proc.stdout.decode('utf-8').split()[0]

    # Determine size of data file in bytes
    size = os.path.getsize(infile)

    # Indeces of _s in file name
    i1 = fileidentifier.find("_")
    i2 = (i1+1) + fileidentifier[i1+1:].find("_")
    i3 = (i2+1) + fileidentifier[i2+1:].find("_")
    i4 = (i3+1) + fileidentifier[i3+1:].find("_")
    i5 = (i4+1) + fileidentifier[i4+1:].find("_")
    ipd1 = fileidentifier.rfind("_")
    campaignshortname = fileidentifier[i1+1:i2]
    datashortname     = fileidentifier[i2+1:i4]
    instrumentname    = fileidentifier[i4+1:i5]
    description       = descriptions[datashortname]
    productiontime    = fileidentifier[ipd1+1:]
    productiondatetime = productiontime[0:4] + "-" + productiontime[4:6] + "-" + productiontime[6:8] + productiontime[8:11] + ":" + productiontime[11:13] + ":" + productiontime[13:] + "Z"

    # Read netCDF file for min/max time, lat, lon
    ncfile = netCDF4.Dataset(infile,"r")
    times  = ncfile.variables["time"][:]
    lats   = ncfile.variables["latitude"][:]
    lons   = ncfile.variables["longitude"][:]
    ncfile.close()

    # Check for 1D or 2D data
    firstdim = len(times)
    try:
        seconddim = len(times[0])
    except TypeError:
        seconddim = 0

    # Loop through data points
    if seconddim == 0:
        minlat  = lats[0]
        maxlat  = lats[0]
        minlon  = lons[0]
        maxlon  = lons[0]
        mintime = times[0]
        maxtime = times[0]
        for i in range(0,firstdim):
            if lats[i] > maxlat:
                maxlat = lats[i]
            if lats[i] < minlat:
                minlat = lats[i]
            if lons[i] > maxlon:
                maxlon = lons[i]
            if lons[i] < minlon:
                minlon = lons[i]
            if times[i] > maxtime:
                maxtime = times[i]
            if times[i] < mintime:
                mintime = times[i]
    else:
        minlat  = lats[0][0]
        maxlat  = lats[0][0]
        minlon  = lons[0][0]
        maxlon  = lons[0][0]
        mintime = times[0][0]
        maxtime = times[0][0]
        for i in range(0,firstdim):
            for j in range(0,seconddim):
                if lats[i][j] > maxlat:
                    maxlat = lats[i][j]
                if lats[i][j] < minlat:
                    minlat = lats[i][j]
                if lons[i][j] > maxlon:
                    maxlon = lons[i][j]
                if lons[i][j] < minlon:
                    minlon = lons[i][j]
                if times[i][j] > maxtime:
                    maxtime = times[i][j]
                if times[i][j] < mintime:
                    mintime = times[i][j]
    
    mindate = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(mintime + REFTIMEUTC))
    maxdate = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(maxtime + REFTIMEUTC))

    xmlname = fileidentifier + ".xml"
    xmlfile = os.path.join(datadir,xmlname)
    if os.path.isfile(xmlfile):
        os.remove(xmlfile)
    ifxml = open(xmlfile,"w")
    ifxml.write("<GranuleMetaDataFile>\n")
    ifxml.write("    <Project>SWOTCalVal</Project>\n")
    ifxml.write("    <FileIdentifier>%s</FileIdentifier>\n" % (fileidentifier))
    ifxml.write("    <CollectionMetaData>\n")
    ifxml.write("        <Campaign>MASS/Lidar: California Crossover</Campaign>\n")
    ifxml.write("        <CampaignShortName>%s</CampaignShortName>\n" % (campaignshortname))
    ifxml.write("        <ShortName>%s</ShortName>\n" % (datashortname))
    ifxml.write("        <InstrumentName>%s</InstrumentName>\n" % (instrumentname))
    ifxml.write("        <Description>%s</Description>\n" % (description))
    ifxml.write("    </CollectionMetaData>\n")
    ifxml.write("    <Extent>\n")
    ifxml.write("        <TemporalExtent>\n")
    ifxml.write("            <StartDateTime>%s</StartDateTime>\n" % (mindate))
    ifxml.write("            <EndDateTime>%s</EndDateTime>\n" % (maxdate))
    ifxml.write("        </TemporalExtent>\n")
    ifxml.write("        <SpatialExtent>\n")
    ifxml.write("            <WestBoundLongitude>%.6f</WestBoundLongitude>\n" % (minlon))
    ifxml.write("            <EastBoundLongitude>%.6f</EastBoundLongitude>\n" % (maxlon))
    ifxml.write("            <SouthBoundLatitude>%.6f</SouthBoundLatitude>\n" % (minlat))
    ifxml.write("            <NorthBoundLatitude>%.6f</NorthBoundLatitude>\n" % (maxlat))
    ifxml.write("        </SpatialExtent>\n")
    ifxml.write("    </Extent>\n")
    ifxml.write("    <DataGranuleMembers>\n")
    ifxml.write("        <DataGranuleMember>\n")
    ifxml.write("            <LocalGranuleID>%s</LocalGranuleID>\n" % (xmlname))
    ifxml.write("            <ProductionDateTime>%s</ProductionDateTime>\n" % (productiondatetime))
    ifxml.write("        </DataGranuleMember>\n")
    ifxml.write("        <DataGranuleMember>\n")
    ifxml.write("            <LocalGranuleID>%s</LocalGranuleID>\n" % (datafile))
    ifxml.write("            <ProductionDateTime>%s</ProductionDateTime>\n" % (productiondatetime))
    ifxml.write("            <Checksum>%s</Checksum>\n" % (md5CheckSum))
    ifxml.write("            <ChecksumType>MD5</ChecksumType>\n")
    ifxml.write("            <Size>%d</Size>\n" % (size))
    ifxml.write("        </DataGranuleMember>\n")
    ifxml.write("    </DataGranuleMembers>\n")
    ifxml.write("</GranuleMetaDataFile>\n")
    ifxml.close()
